#clear the memory
rm(list=ls())

#change directory
setwd("D:/_Travaux/university/Cours_Universite/Supports_de_cours/Informatique/R/Tutoriels/wrapper")

#load the data file
mushroom <- read.table(file="mushroom.txt",header=TRUE,sep="\t")
summary(mushroom)

#partitioning into train and test samples
mushroom.train <- as.data.frame(mushroom[mushroom$statut=="train",c(1:22,24)])
mushroom.test <- as.data.frame(mushroom[mushroom$statut=="test",c(1:22,24)])
print(nrow(mushroom.train))
print(nrow(mushroom.test))

#load the RWeka package
library(RWeka)

#make naive bayes classifier function
NB <- make_Weka_classifier("weka/classifiers/bayes/NaiveBayes")

#training phase
full.model <- NB(classe ~ ., data = mushroom.train)

#evaluation on the test set
test.evaluation <- evaluate_Weka_classifier(full.model,newdata=mushroom.test)
print(test.evaluation)

#callback function
#searching the best predictive attribute
#for one step of the wrapper process
search_wrapper <- function(x,cur.dataset,K){
  #current dataset
  working <- cbind(cur.dataset,x)
  #learning the model
  cur.model <- NB(as.formula(paste(colnames(working)[1],"~ .")),data = working)
  #evaluating the model
  cur.eval <- evaluate_Weka_classifier(cur.model,numFolds = K,seed=100)
  #getting the error rate
  return (cur.eval$details["pctIncorrect"])
}

#programming the whole wrapper framework for feature subset selection
#forward selection process
#dataset is the available data (train set only, we do not use the test set here)
#we assume that the last column is the class attribute
#K is the number of fold in the cross-validation
wrapper <- function(dataset,K=10){
 #number of predictive attribute
 P <- ncol(dataset)-1
 #predictives dataset
 predictives <- dataset[1:(ncol(dataset)-1)]
 #classe attribute
 cur.dataset <- dataset[ncol(dataset)]
 #current formula
 cur.formula <- paste(colnames(dataset)[P+1],"~")
 #ordered index of the selected variable
 output <- c()
 #error rate at each step
 error <- c()
 #the whole process
 for (p in 1:P){
  #error rate for this step
  cur.error <- sapply(predictives,search_wrapper,cur.dataset,K)
  #getting the best error rate 
  id.min <- which.min(cur.error)
  selected.name <- colnames(predictives)[id.min]
  #adding the id. of the column into the selection
  output[p] <- which.min(match(colnames(dataset),selected.name))
  #add the attribute to the current dataset
  cur.dataset <- cbind(cur.dataset,predictives[id.min])
  #removing the attribute to the predictive attributes
  predictives <- predictives[-id.min]
  #filling out the vector of errors
  error[p] <- min(cur.error)
 }
 #return 
 return (list(error=error,output=output))
}

#launching the wrapper process
mushroom.wrapper <- wrapper(mushroom.train)

#plotting the error rate according the number of selected variables
plot(mushroom.wrapper$error,type="b",main="Wrapper process",ylab="Error rate (%)",xlab="# of selected variables")

#getting the number of selected variable
nb.sel <- which.min(mushroom.wrapper$error)

#new dataset -- train and test
new.train <- mushroom.train[,c(mushroom.wrapper$output[1:nb.sel],ncol(mushroom.train))]
new.test <- mushroom.test[,c(mushroom.wrapper$output[1:nb.sel],ncol(mushroom.test))]

#printing the name of the selected variables
print("Selected variables : ")
print(colnames(new.train)[1:nb.sel])

#training and test on the selected variables
new.model <- NB(classe ~., data = new.train)
new.eval <- evaluate_Weka_classifier(new.model,newdata=new.test)
print(new.eval)
